from Globals import DTMLFile
from cgi import escape
import string
from OFS.CopySupport import cookie_path

class proxy:
    """Proxies catalog brains."""

    def __init__(self, object):
        self.object = object

    def __getattr__(self, name):
        if hasattr(self.object.aq_base, name):
            return getattr(self.object, name)
        else:
            return getattr(self.object.getObject(), name)

class lazy:
    """Wrapper around catalog search results."""

    def __init__(self, results):
        self.results_ = results

    def __getattr__(self, name):
        return getattr(self.results_, name)

    def __getitem__(self, index):
        return proxy(self.results_[index])

    def __getslice__(self, start, end):
        if end > len(self):
            end = len(self)
        return map(self.__getitem__, range(start, end))

class base:
    """Mixin class used to provide common functionality."""

    search = DTMLFile('issue_search', globals())
    remote = DTMLFile('remote', globals())
    issue_tree = DTMLFile('issue_tree', globals())
    issue_listing = DTMLFile('issue_listing', globals())
    standard_html_footer = DTMLFile('standard_html_footer', globals())
    settings = DTMLFile('settings', globals())

    def render_breadcrumbs(self):
        """Renders breadcrumbs."""
        parents = self.REQUEST['PARENTS'][:]
        breadcrumbs = []
        for parent in parents:
            breadcrumbs.append("""<a href="%s">%s</a>""" % \
             (parent.absolute_url(), escape(parent.get_title())))
            if parent.meta_type == 'Issue Manager':
                break
        breadcrumbs.reverse()
        return """<div id="breadcrumbs">""" + \
               string.join(breadcrumbs, ' :: ') + \
               """</div>"""

    def get_title(self):
        """Returns a descriptive title."""
        if self.title:
            title = self.title
        else:
            title = self.meta_type
        return title

    def get_search_results(self):
        """Returns search results, based on REQUEST."""
        search = self.REQUEST['search_string']
        return self.Catalog(get_all_text=search)

    def get_all_text(self):
        """Returns all text on the object."""
        return self.title + " " + self.contents

    def get_issues(self):
        """Returns a list of the (non deleted) Issues contained."""
        states = ['open', 'closed']
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            states.append('deleted')
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            state=states)

    def get_open_issues(self):
        """Returns a list of the (non deleted), open Issues contained."""
        states = ['open']
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            states.append('deleted')
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            state=states)

    def get_number_of_issues(self):
        """Returns the number of Issues contained."""
        return len(self.get_issues())

    def manage_cut(self, ids=None):
        """Cuts selected objects."""
        copy_data = self.manage_cutObjects(ids=ids)
        self.REQUEST.RESPONSE.setCookie('__cp', copy_data,
                                        path='%s' % cookie_path(self.REQUEST))
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    def manage_paste(self):
        """Pastes cut objects."""
        self.manage_pasteObjects(cb_copy_data=self.REQUEST['__cp'])
        self.REQUEST['RESPONSE'].setCookie('__cp', 'deleted',
                                      path='%s' % cookie_path(self.REQUEST),
                                      expires='Wed, 31-Dec-97 23:59:59 GMT')
        self.REQUEST['__cp'] = None
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    def manage_delete(self, ids=[]):
        """Deletes the specified objects."""
        for id in ids:
            self[id].state = 'deleted'
            self[id].index_object()
            self[id].manage_delete(ids=self[id].objectIds())
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    def manage_undelete(self, ids=[]):
        """Undeletes the specified objects."""
        for id in ids:
            self[id].state = 'open'
            self[id].index_object()
            self[id].manage_delete(ids=self[id].objectIds())
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    def get_parent_url(self):
        """Returns the parents URL."""
        return self.getParentNode().absolute_url(relative=1)

    def catalog_search(self, *arguments, **keywords):
        """Does a catalog search, returning proxied objects."""
        return lazy(self.Catalog(*arguments, **keywords))
