from Globals import DTMLFile, MessageDialog, Persistent
import OFS
import Acquisition
import AccessControl.Role
from Products import ZCatalog
import base
from StructuredText.StructuredText import HTML
from cgi import escape
import string

def manage_add_issue(self, id=None, title='', contents='', format='',
                     state='', type='', referrer='', REQUEST=None):
    """Add an Issue."""
    if id is None:
        id = self.get_unique_id()
    issue_ = issue(id, title, contents, format, state, type)
    self._setObject(id, issue_)
    try:
        self.index_object()
    except AttributeError:
        pass
    if referrer and REQUEST is not None:
        REQUEST.RESPONSE.redirect(referrer)
    else:
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)
        else:
            return issue_

def manage_add_issue_edit(self, id=None, title='', contents='', referrer='',
                          state='', type='', REQUEST=None):
    """Adds an issue and redirects to the edit page."""
    issue_ = manage_add_issue(self, id=id, title=title, contents=contents,
                              referrer=referrer, state=state, type=type)
    REQUEST.RESPONSE.redirect(self.absolute_url() + '/' + issue_.id + '/issue_edit')

class issue(
    ZCatalog.CatalogAwareness.CatalogAware,
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    base.base
    ): 
    """Issue class.

    An issue contains information relating to a task, an
    idea, or whatever.
    """

    meta_type = 'Issue'
    state = 'open'
    type = 'undefined'
    types = ( 'container', 'critical problem', 'goal', 'idea', 'info',
             'problem', 'solution', 'choice', 'undefined')

    all_meta_types = [
        {'visibility': 'Global',
         'interfaces': [],
         'action': 'manage_add_issue',
         'permission': 'Add Issues',
         'name': 'Issue',
         'product': 'Issue Manager',
         'instance': ''}
    ]

    manage_options = (OFS.Folder.Folder.manage_options[0],) + \
        (
            {'label': 'View',       'action': ''},
            {'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=(
	('View management screens', ('manage_tabs','manage_main',
                                     'manage_workspace')),
	('Cut and paste issues', ('manage_cut', 'manage_paste')),
	('Delete issues', ('manage_delete',)),
	('Change permissions', ('manage_access',)),
	('Edit issues', ('issue_edit',)),
	('View issues', ('',)),
    )

    def __init__(self, id, title='', contents='', format='', state='', type=''): 
        """initialise a new instance of Issue"""
        self.id = id
        self.title = title
        self.contents = contents
        self.format = format or 'text'
        self.state = state or 'open'
        self.type = type or 'undefined'

    def render_stx_as_html(self):
        """Renders the contents as STX."""
        return HTML(self.contents, level=3, header=0)

    def render_contents_as_text(self):
        """Renders the contents as text, keeping whitespace indentation."""
        contents = self.contents
        contents = contents.replace('\r\n', '\n')
        contents = contents.replace('\n\r', '\n')
        contents = escape(contents)
        lines = contents.split('\n')
        new_lines = []
        for line in lines:
            if not line:
                new_lines.append(line)
                continue
            new_line = ''
            if line[0] == ' ':
                for character in line:
                    if character == ' ':
                        new_line += '&nbsp;'
                    else:
                        new_line += character
            else:
                new_line = line
            new_lines.append(new_line)
        return string.join(new_lines, '<br />')

    def manage_edit(self, title, contents, format, state, type, REQUEST=None):
        """Edits the object."""
        self.title = title
        self.contents = contents
        self.format = format
        self.state = state
        self.type = type
        self.index_object()
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    def get_title(self):
        """Returns a descriptive title."""
        if self.title:
            title = self.title
        else:
            title = self.meta_type
        if self.state == 'deleted':
            title += " [Deleted] "
        return title

    def render_html_title_and_link(self):
        """Renders a coloured link, depending on the type (and state) of the Issue."""
        color = ''
        if self.state == 'open':
            if self.type == 'critical problem':
                color = 'red'
            elif self.type == 'problem':
                color = 'yellow'
            else:
                color = 'green'
        else:
            color = 'green'
        link = """<a href="%s" %s>%s</a>"""
        title = escape(self.get_title())
        if color:
            link = """<a href="%s" class="%s">%s</a>"""
            link = link % (self.absolute_url(),
                           color, title)
        else:
            link = """<a href="%s">%s</a>"""
            link = link % (self.absolute_url(), title)
        return link

    index_html = DTMLFile('index', globals())
    manage_add_issue = manage_add_issue
    manage_add_issue_edit = manage_add_issue_edit
    issue_edit = DTMLFile('issue_edit', globals())
    render_contents = DTMLFile('issue_render_contents', globals())

    def render_contents(self):
        """Renders the contents."""
        if self.format == 'stx':
            if self.REQUEST.has_key('expand_text') and self.REQUEST['expand_text']:
                return self.render_stx_as_html() + \
		' <a href="?expand_text:int=0">Less..</a>'
            else:
                contents = self.render_stx_as_html()
                if len(contents) > self.size:
                    contents += ' <a href="%s?expand_text:int=1">More..</a>' % \
                      self.absolute_url()
                return contents
        else:
            if self.REQUEST.has_key('expand_text') and self.REQUEST['expand_text']:
                return self.render_contents_as_text() + \
		  ' <a href="?expand_text:int=0">Less..</a>'
            else:
                contents = self.render_contents_as_text()
                if len(contents) > self.size:
                    contents += ' <a href="%s?expand_text:int=1">More..</a>' % \
                      self.absolute_url()
                return contents
